
;*****************************************************************************
;                                                                            *
;    Fichier requis: P16F628.inc                                             *
;                                                                            *
;                                                                            *
;                                                                            *
;*****************************************************************************
;                                                                            *
;    Notes:  essai de sauvegarde sur l' eeprom 16f628 , horloge interne      *
;                                                                            *
;     led sur RB0 , la vitesse de clignotement est sauvegarder dans la 2em   *
;     adresse de l' eeprom , pour changer la vitesse sw1 sur ra0             *
;                                                                            *
;*****************************************************************************


	LIST      p=16F628            ; Dfinition de processeur
	#include <p16F628.inc>        ; fichier include

		__CONFIG  _CP_OFF & _DATA_CP_OFF & _LVP_OFF & _BODEN_OFF & _MCLRE_ON & _PWRTE_ON & _WDT_OFF & _INTRC_OSC_NOCLKOUT



;	errorlevel -302


; '__CONFIG' prcise les paramtres encods dans le processeur au moment de
; la programmation du processeur. Les dfinitions sont dans le fichier include.
; Choisir une valeur par groupe.
; Voici les valeurs et leurs dfinitions :

; Protection du programme
; -----------------------
;_CP_ALL		protection totale
;_CP_75 		protection de 1000  1FFF
;_CP_50			protection de 1F00  1FFF
;_CP_OFF		Pas de protection


; Protection de la EEprom
; -----------------------

;_DATA_CP_ON		Mmoire EEprom protge
;_DATA_CP_OFF		Mmoire EEprom dprotge

; Programmation sur circuit
; -------------------------

;_LVP_ON		RB3 permet la programmation srie de la PIC
;_LVP_OFF		RB3 en utilisation normale

; Reset de la PIC si tension <4V
; ------------------------------
; _BODEN_ON		Reset tension en service
;			Valide PWRTE_ON automatiquement
; _BODEN_OFF		Reset tension hors service

;Utilisation de la pin MCLR
; ------------------------------
;_MCLRE_ON		RA5/MCLR est utilise pour le reset
;
;_MCLRE_OFF		RA5/MCLR est utilise comme une entree/sortie

; Retard  la mise sous tension
; -----------------------------

;_PWRTE_OFF		Dmarrage rapide
;_PWRTE_ON		Dmarrage temporis

; Watchdog
; --------

;_WDT_ON		Watchdog en service
;_WDT_OFF		Watchdog hors service

; Oscillateur
; -----------
;_ER_OSC_CLKOUT 	clckout sur RA6 resistance externe surRA7             
;_ER_OSC_NOCLKOUT	I/O sur RA6 resistance externe sur RA7
;			(1K*10MHz-20K*5.3MHz-100k*1.6MHz-220M*800KHz-1M*200KHz)   
;_INTRC_OSC_CLKOUT	oscillateur interne clkout sur RA6 I/O sur RA7 (4MHz)    
;_INTRC_OSC_NOCLKOUT	oscillateur interne I/O sur RA6 et RA7 (4MHz)        
;_EXTCLK_OSC		signal clock fournit au pic sur RA7 I/O sur RA6                
;_LP_OSC 		Oscillateur basse vitesse (?<F<200Khz)        
;_XT_OSC                Oscilateur moyenne vitesse (0,1MHz<F<4Mhz)     
;_HS_OSC 		Oscillateur haute vitesse (1Mhz<F<20Mhz)


;*****************************************************************************
;                               ASSIGNATIONS SYSTEME                         *
;*****************************************************************************

; REGISTRE OPTION_REG (configuration)
; -----------------------------------
OPTIONVAL	EQU	B'00000000'
			; RBPU      b7 : 1= Rsistance rappel +5V hors service
			; INTEDG    b6 : 1= Interrupt sur front montant de RB0
			;                0= Interrupt sur front descendant de RB0
			; TOCS      b5 : 1= source clock = transition sur RA4
			;                0= horloge interne
			; TOSE      b4 : 1= Slection front descendant RA4(si B5=1)
			;                0= Slection front montant RA4
			; PSA       b3 : 1= Assignation prdiviseur sur Watchdog
			;                0= Assignation prdiviseur sur Tmr0
			; PS2/PS0   b2/b0 valeur du prdiviseur
         ;           000 =  1/1 (watchdog) ou 1/2 (tmr0)
			;           001 =  1/2          1/4
			;           010 =  1/4		     1/8
			;           011 =  1/8		     1/16
			;           100 =  1/16		     1/32
			;           101 =  1/32		     1/64
			;           110 =  1/64		     1/128
			;           111 =  1/128	     1/256


; REGISTRE INTCON (contrle interruptions standard)
; -------------------------------------------------
INTCONVAL	EQU	B'00000000'
			; GIE       b7 : masque autorisation gnrale interrupt
                        ;                ne pas mettre ce bit  1 ici
                        ;                sera mis en temps utile
			; PEIE      b6 : masque autorisation gnrale priphriques
			; T0IE      b5 : masque interruption tmr0
			; INTE      b4 : masque interuption RB0/Int
			; RBIE      b3 : masque interruption RB4/RB7
			; T0IF      b2 : flag tmr0
			; INTF      b1 : flag RB0/Int
			; RBIF      b0 : flag interruption RB4/RB7

; REGISTRE PIE1 (contrle interruptions priphriques)
; ----------------------------------------------------
PIE1VAL		EQU	B'00000000'
			; EEIE      b7 : masque interrupt criture EEPROM
			; CMIE      b6 : masque interrupt comparateur
			; RCIE      b5 : masque interrupt rception USART
			; TXIE      b4 : masque interrupt transmission USART
			; RESERVED  b3 : rserv, laisser  0
			; CCP1IE    b2 : masque interrupt CCP1
			; TMR2IE    b1 : masque interrupt TMR2 = PR2
			; TMR1IE    b0 : masque interrupt dbordement tmr1


; REGISTRE CMCON (COMPARATEURS)
; ------------------------------------
CMCONVAL	EQU	B'00000111' 
			; C2OUT	    b7 :sortie comparateur2
			; C1OUT     b6 :sortie comparateur1
			; C2INV     b5 :inverseur comparateur2
			; C1INV     b4 :inverseur comparateur1
			; CIS       b3 :selection entree des comparateurs
			; CM2-CM0   b2-0 :mode des comparateurs
			;configurer le mode 111 pour utiliser RA0 RA1 RA2 RA3 en I/O

; REGISTRE VRCON (voltage reference module)
; ------------------------------------
VRCONVAL	EQU	B'00000000'
			; VREN      b7 :Validation du module
			; VROE      b6 :Sortie sur RA2
			; VRR       b5 :Choix de la plage 
			; RESERVED  b4 : rserv, laisser  0
			; VR3-VR0   b3-0 :Choix de la valeur dans la plage 
					; si VRR=1 VREF=(VR[3:0]/24)*Vdd
					; si VRR=0 VREF=Vdd/4(VR[3:0]/32)*Vdd

; DIRECTION DES PORTS I/O
; -----------------------

DIRPORTA	EQU	B'00000001'	; Direction PORTA (1=entre)
DIRPORTB	EQU	B'00000000'	; Direction PORTB


;*****************************************************************************
;                           ASSIGNATIONS PROGRAMME                           *
;*****************************************************************************


;*****************************************************************************
;                                  DEFINE                                    *
;*****************************************************************************


;*****************************************************************************
;                             MACRO                                          *
;*****************************************************************************

			; Changement de banques
			; ----------------------

BANK0	macro				; passer en banque0
		bcf	STATUS,RP0
		bcf	STATUS,RP1
	endm

BANK1	macro				; passer en banque1
		bsf	STATUS,RP0
		bcf	STATUS,RP1
	endm

BANK2	macro				; passer en banque2
		bcf	STATUS,RP0
		bsf	STATUS,RP1
	endm

BANK3	macro				; passer en banque3
		bsf	STATUS,RP0
		bsf	STATUS,RP1
	endm



			; oprations en mmoire eeprom
			; -----------------------------

REEPROM macro	adeeprom		; lire eeprom (adresse dans adeeprom & rsultat en w)
	movlw	adeeprom				; charger adresse eeprom (passage de la valeur en litteral)
	clrwdt						; reset watchdog
	BANK1							; passer en banque1
	movwf	EEADR					; pointer sur adresse eeprom
	bsf	EECON1,RD			; ordre de lecture
	movf	EEDATA,w				; charger valeur lue
	BANK0							; passer en banque0
	endm


WEEPROM	macro	addwrite	  	; la donne se trouve dans W
	LOCAL	loop
	BANK1							; passer en banque1
	movwf	EEDATA				; placer data dans registre
	movlw	addwrite				; charger adresse d'criture (passage de la valeur en litteral)
	movwf	EEADR					; placer dans registre
	bsf	EECON1 , WREN		; autoriser accs criture
	bcf	INTCON , GIE		; interdire interruptions
	movlw	0x55					; charger 0x55
	movwf	EECON2				; envoyer commande
	movlw	0xAA					; charger 0xAA
	movwf	EECON2				; envoyer commande
	bsf	EECON1 , WR			; lancer cycle d'criture
	bsf	INTCON , GIE		; rautoriser interruptions
loop
	btfsc	EECON1 , WR			; tester si criture termine
	goto	loop			; non, attendre
	bcf	EECON1 , WREN		; verrouiller prochaine criture
	BANK0							; passer en banque0
	endm





;*****************************************************************************
;                        VARIABLES BANQUE 0                                  *
;*****************************************************************************

; Zone de 80 bytes
; ----------------

	CBLOCK	0x20				; Dbut de la zone (0x20  0x6F)
			VIT		:1
			Delay_1  :1	   
			Delay_2  :1   
			Delay_3	:1

        ENDC					; Fin de la zone                        


;*****************************************************************************
;                      VARIABLES ZONE COMMUNE                                *
;*****************************************************************************

; Zone de 16 bytes
; ----------------

	CBLOCK 0x70					; Dbut de la zone (0x70  0x7F)
	w_temp 		: 1			; Sauvegarde registre W
	status_temp : 1			; sauvegarde registre STATUS
	FSR_temp 	: 1			; sauvegarde FSR (si indirect en interrupt)
	adeeprom		: 1
	addwrite		: 1

	ENDC

;*****************************************************************************
;                        VARIABLES BANQUE 1                                  *
;*****************************************************************************

; Zone de 80 bytes
; ----------------

;	CBLOCK	0xA0				; Dbut de la zone (0xA0  0xEF)

;	ENDC							; Fin de la zone                        

;*****************************************************************************
;                        VARIABLES BANQUE 2                                  *
;*****************************************************************************

; Zone de 48 bytes
; ----------------

;	CBLOCK	0x120				; Dbut de la zone (0x120  0x14F)

;	ENDC							; Fin de la zone                        

       

;*****************************************************************************
;                      DEMARRAGE SUR RESET                                   *
;*****************************************************************************

	org 0x000 					; Adresse de dpart aprs reset
  	goto     init				; Initialiser


; ////////////////////////////////////////////////////////////////////////////

;                         I N T E R R U P T I O N S

; ////////////////////////////////////////////////////////////////////////////

;*****************************************************************************
;                     ROUTINE INTERRUPTION                                   *
;*****************************************************************************
;-----------------------------------------------------------------------------
; Si on n'utilise pas l'adressage indirect dans les interrupts, on se passera
; de sauvegarder FSR
;-----------------------------------------------------------------------------
			;sauvegarder registres	
			;---------------------
	org 0x004					; adresse d'interruption
	movwf   w_temp  			; sauver registre W
	swapf	STATUS,w				; swap status avec rsultat dans w
	movwf	status_temp			; sauver status swapp
	movf	FSR , w				; charger FSR
	movwf	FSR_temp				; sauvegarder FSR
	BANK0							; passer en banque0

			; switch vers diffrentes interrupts
			; inverser ordre pour modifier priorits
			; mais attention alors au test PEIE
			; effacer les inutiles
			;----------------------------------------
	
			; Interruption TMR0
			; -----------------

	btfsc	INTCON,T0IE			; tester si interrupt timer autorise
	btfss	INTCON,T0IF			; oui, tester si interrupt timer en cours
	goto 	intsw1				; non test suivant
	call	inttmr0				; oui, traiter interrupt tmr0
	bcf	INTCON,T0IF			; effacer flag interrupt tmr0
	goto	restorereg			; et fin d'interruption
				; SUPPRIMER CETTE LIGNE POUR
				; TRAITER PLUSIEURS INTERRUPT
				; EN 1 SEULE FOIS

			; Interruption RB0/INT
			; --------------------
intsw1
	btfsc	INTCON,INTE			; tester si interrupt RB0 autorise
	btfss	INTCON,INTF			; oui, tester si interrupt RB0 en cours
	goto 	intsw2				; non sauter au test suivant
	call	intrb0				; oui, traiter interrupt RB0
	bcf	INTCON,INTF			; effacer flag interupt RB0
	goto	restorereg			; et fin d'interruption
				; SUPPRIMER CETTE LIGNE POUR
				; TRAITER PLUSIEURS INTERRUPT
				; EN 1 SEULE FOIS

			; interruption RB4/RB7
			; --------------------
intsw2
	btfsc	INTCON,RBIE			; tester si interrupt RB4/7 autorise
	btfss	INTCON,RBIF			; oui, tester si interrupt RB4/7 en cours
	goto 	intsw3				; non sauter
	call	intrb4				; oui, traiter interrupt RB4/7
	bcf	INTCON,RBIF			; effacer flag interupt RB4/7
	goto	restorereg			; et fin d'interrupt

			; dtection interruptions priphriques
			; le test peut tre supprim si une seule
			; interrupt est traite  la fois
			; --------------------------------------
intsw3
	btfss	INTCON,PEIE			; tester interruption priphrique autorise
	goto	restorereg			; non, fin d'interruption

	

			; Interruption rception USART
			; ----------------------------
intsw4
	BANK1							; slectionner banque1
	btfss	PIE1,RCIE			; tester si interrupt autorise
	goto 	intsw5				; non sauter
	BANK0							; oui, slectionner banque0
	btfss	PIR1,RCIF			; oui, tester si interrupt en cours
	goto 	intsw5				; non sauter
	call	intrc					; oui, traiter interrupt
				; LE FLAG NE DOIT PAS ETRE REMIS A 0
				; C'EST LA LECTURE DE RCREG QUI LE PROVOQUE
	goto	restorereg		; et fin d'interrupt

			; Interruption transmission USART
			; -------------------------------
intsw5
	BANK1							; slectionner banque1
	btfss	PIE1,TXIE			; tester si interrupt autorise
	goto 	intsw6				; non sauter
	BANK0							; oui, slectionner banque0
	btfss	PIR1,TXIF			; oui, tester si interrupt en cours
	goto 	intsw6				; non sauter
	call	inttx					; oui, traiter interrupt
				; LE FLAG NE DOIT PAS ETRE REMIS A 0
				; C'EST L'ECRITURE DE TXREG QUI LE PROVOQUE
	goto	restorereg		; et fin d'interrupt

			; Interruption comparateurs
			; -------------------------
intsw6
	BANK1							; slectionner banque1
	btfss	PIE1,CMIE			; tester si interrupt autorise
	goto 	intsw7				; non sauter
	BANK0							; oui, slectionner banque0
	btfss	PIR1,CMIF			; oui, tester si interrupt en cours
	goto 	intsw7				; non sauter
	call	intcmp				; oui, traiter interrupt
				; LE FLAG NE DOIT PAS ETRE REMIS A 0
				; C'EST L'ECRITURE DE TXREG QUI LE PROVOQUE
	goto	restorereg		; et fin d'interrupt
	

			; Interruption CCP1
			; -----------------
intsw7
	BANK1							; slectionner banque1
	btfss	PIE1,CCP1IE			; tester si interrupt autorise
	goto 	intsw8				; non sauter
	BANK0							; oui, slectionner banque0
	btfss	PIR1,CCP1IF			; oui, tester si interrupt en cours
	goto 	intsw8				; non sauter
	call	intccp1				; oui, traiter interrupt
	bcf	PIR1,CCP1IF			; effacer flag interupt 
	goto	restorereg			; et fin d'interrupt


			; Interruption TMR2
			; -----------------
intsw8
	BANK1							; slectionner banque1
	btfss	PIE1,TMR2IE			; tester si interrupt autorise
	goto 	intsw9				; non sauter
	BANK0							; oui, slectionner banque0
	btfss	PIR1,TMR2IF			; oui, tester si interrupt en cours
	goto 	intsw9				; non sauter
	call	inttmr2				; oui, traiter interrupt
	bcf	PIR1,TMR2IF			; effacer flag interupt 
	goto	restorereg			; et fin d'interrupt

			; Interruption TMR1
			; -----------------
intsw9
	BANK1							; slectionner banque1
	btfss	PIE1,TMR1IE			; tester si interrupt autorise
	goto 	intswA				; non sauter
	BANK0							; oui, slectionner banque0
	btfss	PIR1,TMR1IF			; oui, tester si interrupt en cours
	goto 	intswA				; non sauter
	call	inttmr1				; oui, traiter interrupt
	bcf	PIR1,TMR1IF			; effacer flag interupt 
	goto	restorereg			; et fin d'interrupt

			; Interruption EEPROM
			; -------------------
intswA
	BANK1							; slectionner banque1
	btfss	PIE1,EEIE			; tester si interrupt autorise
	goto 	restorereg			; non sauter
	BANK0							; oui, slectionner banque0
	btfss	PIR1,EEIF			; oui, tester si interrupt en cours
	goto 	restorereg			; non sauter
	call	inteprom				; oui, traiter interrupt
	bcf	PIR1,EEIF			; effacer flag interupt 
	goto	restorereg			; et fin d'interrupt

	
			;restaurer registres
			;-------------------
restorereg
	movf	FSR_temp , w		; charger FSR sauv
	movwf	FSR					; restaurer FSR
	swapf	status_temp,w		; swap ancien status, rsultat dans w
	movwf   STATUS				; restaurer status
	swapf   w_temp,f			; Inversion L et H de l'ancien W
                       		; sans modifier Z
	swapf   w_temp,w  		; Rinversion de L et H dans W
									; W restaur sans modifier status
	retfie  						; return from interrupt

;*****************************************************************************
;                        INTERRUPTION TIMER 0                                *
;*****************************************************************************
inttmr0
	return						; fin d'interruption timer
					; peut tre remplac par 
					; retlw pour retour code d'erreur

;*****************************************************************************
;                        INTERRUPTION RB0/INT                                *
;*****************************************************************************
intrb0
	return						; fin d'interruption RB0/INT
					; peut tre remplac par 
					; retlw pour retour code d'erreur

;*****************************************************************************
;                         INTERRUPTION RB0/RB4                               *
;*****************************************************************************
intrb4
	return						; fin d'interruption RB0/RB4
					; peut tre remplac par 
					; retlw pour retour code d'erreur

;*****************************************************************************
;                     INTERRUPTION RECEPTION USART                           *
;*****************************************************************************
intrc
	return						; fin d'interruption
					; peut tre remplac par 
					; retlw pour retour code d'erreur

;*****************************************************************************
;                     INTERRUPTION TRANSMISSION USART                        *
;*****************************************************************************
inttx
	return						; fin d'interruption
					; peut tre remplac par 
					; retlw pour retour code d'erreur

;*****************************************************************************
;                       INTERRUPTION CCP1                                    *
;*****************************************************************************
intccp1
	return						; fin d'interruption
					; peut tre remplac par 
					; retlw pour retour code d'erreur
;****************************************************************************
;                      INTERUPTION COMPARATEUR                              *
;****************************************************************************
intcmp
	return						; fin d'interruption
					; peut tre remplac par 
					; retlw pour retour code d'erreur
;*****************************************************************************
;                     INTERRUPTION TIMER 2                                   *
;*****************************************************************************
inttmr2
	return						; fin d'interruption
					; peut tre remplac par 
					; retlw pour retour code d'erreur

;*****************************************************************************
;                     INTERRUPTION TIMER 1                                   *
;*****************************************************************************
inttmr1
	return						; fin d'interruption
					; peut tre remplac par 
					; retlw pour retour code d'erreur

;*****************************************************************************
;                     INTERRUPTION EEPROM                                    *
;*****************************************************************************
inteprom
	return						; fin d'interruption
					; peut tre remplac par 
					; retlw pour retour code d'erreur

;*****************************************************************************
;                     routines delay                                         *
;*****************************************************************************

;Delay : cette fonction ralise une temporisation variable, selon la valeur de l'accumulateur
;au moment de son appel x 10ms.si w=1 delay =10ms

Delay		movwf		Delay_1
Dloop		decfsz	Delay_3,f
			goto		Dloop
			decfsz	Delay_2,f
			goto		Dloop
			decfsz	Delay_1,f
			goto		Dloop
			return




; ////////////////////////////////////////////////////////////////////////////

;                           P R O G R A M M E

; ////////////////////////////////////////////////////////////////////////////

;*****************************************************************************
;                          INITIALISATIONS                                   *
;*****************************************************************************
init

			; initialisation PORTS (banque 0 et 1)
			; ------------------------------------
	BANK0							; slectionner banque0
	clrf	PORTA					; Sorties PORTA  0
	clrf	PORTB					; sorties PORTB  0
	BANK1							; passer en banque1
	movlw	DIRPORTA				; Direction PORTA
	movwf	TRISA					; criture dans registre direction
	movlw	DIRPORTB				; Direction PORTB
	movwf	TRISB					; criture dans registre direction

			; Registre d'options (banque 1)
			; -----------------------------
	movlw	OPTIONVAL			; charger masque
	movwf	OPTION_REG			; initialiser registre option

			; registres interruptions (banque 1)
			; ----------------------------------
	movlw	INTCONVAL			; charger valeur registre interruption
	movwf	INTCON				; initialiser interruptions
	movlw	PIE1VAL				; Initialiser registre 
	movwf	PIE1					; interruptions priphriques 1
	movlw	VRCONVAL				; charger valeur registre interruption
	movwf	VRCON					; initialiser interruptions
	BANK0							; slectionner banque 0
	movlw	CMCONVAL				; charger valeur registre interruption
	movwf	CMCON					; initialiser interruptions

			; Effacer RAM banque 0
			; ---------------------
	BANK0							; slectionner banque 0
	movlw	0x20					; initialisation pointeur
	movwf	FSR		 			; d'adressage indirect
init1
	clrf	INDF					; effacer ram
	incf	FSR,f					; pointer sur suivant
	btfss	FSR,7					; tester si fin zone atteinte (>7F)
	goto	init1					; non, boucler

			; autoriser interruptions (banque 0)
			; ----------------------------------
	clrf	PIR1					; effacer flags 1
	bsf	INTCON,GIE			; valider interruptions
	goto	START					; programme principal


;*****************************************************************************
;                      PROGRAMME PRINCIPAL                                   *
;*****************************************************************************

START		
		REEPROM	0x02					; lire emplacement eeprom 0x02

			movwf		VIT				; placer dans VIT

DEBUT
			btfsc  	PORTA, 0			; Test de l'tat de SW1 reglage
			goto		BOUCLE			; si ouvert aller boucle
			goto		REGLAGE			; si appuie aller vers reglage
 			
BOUCLE	movlw		b'00000001'		; allume led sur RB0
			movwf		PORTB
			movf		VIT,w				; tempo en fonction de VIT
			call		Delay
			movlw		b'00000000'		; eteind led sur RB0
			movwf		PORTB
			movf		VIT,w				; tempo en fonction de VIT
			call		Delay
			goto		DEBUT				; retour DEBUT

REGLAGE	
			movlw		0x0a
		WEEPROM	0x02					; crire  l'adresse 0x02

			movlw 	0x05				; tempo  de 500ms
	   	call 		Delay		
			goto		DEBUT


;*****************************************************************************
;                   DECLARATIONS DE LA ZONE EEPROM                           *
;*****************************************************************************
	org 0x2100					; adresse dbut zone eeprom
	DE	0x00
	DE	0x00
	DE	0x01


	END 							; directive fin de programme

